# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
newproperty(:security_definer) do
  include EasyType
  include PuppetX::EnterpriseModules::Postgres::Access

  desc <<-EOD
    specified if function has security definer rights.
  EOD

  newvalues(true, false)
  data_type('Optional[Boolean]')

  after_apply do
    statement = if value.to_s == 'true'
                  'definer'
                else
                  'invoker'
                end
    output = sql("alter function #{resource.function_name} security #{statement}", { :instance => resource[:instance], :db => resource.db, :parse => false })
    Puppet.debug(output) if resource.logoutput.to_s == 'true'
  end

  to_translate_to_resource do |raw_resource|
    raw_resource.column_data('security_definer') == 't'
  end
end
