# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
newproperty(:data) do
  include EasyType

  desc <<-EOD
    The data in the row specified as an hash.
  EOD

  data_type('Optional[Hash]')

  to_translate_to_resource do |raw_resource|
    data = raw_resource.dup
    data.delete('SID')
    data.to_h
  end

  #
  # because the order may differ, but they are still the same,
  # to decide if they are equal, first do a sort on is and should
  #
  # rubocop: disable Naming/MethodParameterName
  def insync?(is)
    resource[:ensure].to_s == 'updated' ? is == should : true
  end
  # rubocop: enable Naming/MethodParameterName

  def validate(value)
    fail "pg_record['#{resource[:name]}'] data keyword must be passed a Hash" unless value.is_a?(Hash)

    value.each_key do |key|
      fail "pg_record['#{resource[:name]}'] data keyword must be passed a Hash with string keys" unless key.is_a?(String)
    end
  end

  # rubocop: disable Style/MultilineTernaryOperator
  def change_to_s(from, to)
    difference = Hash[*(from.size > to.size \
      ? from.to_a - to.to_a \
      : to.to_a - from.to_a
                       ).flatten]
    difference.keys.collect { |k| "Changed: #{k} from #{from[k]} to #{to[k]}" }.join(',')
  end
  # rubocop: enable Style/MultilineTernaryOperator
end
